@@ -88,7 +88,7 @@ C CODE BANNER
 # RemezFilterType:
 #    Type I and II symmetric linear phase: neg==0   (filter_type==bandpass)
 #    Type III and IV negative symmetric linear phase: neg==1   (filter_type==hilbert or differentiator)
-@enum RemezFilterType filter_type_bandpass=1 filter_type_differentiator=2 filter_type_hilbert=3
+@enum RemezFilterType filter_type_bandpass filter_type_differentiator filter_type_hilbert
 
 
 
@@ -212,10 +212,10 @@ function freq_eval(xf, x::AbstractVector, y::AbstractVector, ad::AbstractVector)
     d = 0.0
     p = 0.0
 
-    for j = 1:length(ad)
+    for j in eachindex(ad)
         c = ad[j] / (xf - x[j])
         d += c
-        p += c * y[j]
+        p = muladd(c, y[j], p)
     end
 
     p/d
@@ -439,8 +439,8 @@ function remez(numtaps::Integer, band_defs;
     
         #
         # Start next iteration
-        #        
-      @label L100
+        #
+    #   @label L100
         iext[nzz] = ngrid + 1
         niter += 1
         if niter > maxiter
@@ -449,7 +449,9 @@ function remez(numtaps::Integer, band_defs;
             break
         end
 
-        x[1:nz] = grid[iext[1:nz]]
+        for j = 1:nz
+            x[j] = grid[iext[j]]
+        end
 
         for j = 1 : nz
             ad[j] = lagrange_interp(j, nz, jet, x)
@@ -460,8 +462,8 @@ function remez(numtaps::Integer, band_defs;
         k = 1
         for j = 1 : nz
             l = iext[j]
-            dnum += ad[j] * des[l]
-            dden += k * ad[j] / wt[l]
+            dnum = muladd(ad[j], des[l], dnum)
+            dden = muladd(k, ad[j] / wt[l], dden)
             k = -k
         end
         dev = dnum / dden
@@ -495,6 +497,8 @@ function remez(numtaps::Integer, band_defs;
         nut = -nu
         j = 1
 
+        local comp
+
       @label L200
         j == nzz && (ynz = comp)   # equivalent to "if (j == nzz) ynz = comp; end"
         j >= nzz && @goto L300
@@ -604,13 +608,13 @@ function remez(numtaps::Integer, band_defs;
             iext[nzz-j] = iext[nz-j]
         end
         iext[1] = k1
-        @goto L100
+        continue    # @goto L100
       @label L350
         for j = 1:nz
             iext[j] = iext[j+1]
         end
 
-        @goto L100
+        continue    # @goto L100
       @label L370
 
 
@@ -669,9 +673,9 @@ function remez(numtaps::Integer, band_defs;
     for j = 1 : nfcns
         dtemp = 0.0
         for k = 1 : nm1
-            dtemp += a[k+1] * cospi(2 * (j-1) * delf * k)
+            dtemp = muladd(a[k+1], cospi(2 * (j-1) * delf * k), dtemp)
         end
-        alpha[j] = 2.0 * dtemp + a[1]
+        alpha[j] = 2dtemp + a[1]
     end
 
     for j = 2 : nfcns
@@ -680,7 +684,7 @@ function remez(numtaps::Integer, band_defs;
     alpha[1] *= delf
 
     if !full_grid
-        p[1] = 2.0*alpha[nfcns]*bb+alpha[nm1]
+        p[1] = muladd(2alpha[nfcns], bb, alpha[nm1])
         p[2] = 2.0*aa*alpha[nfcns]
         q[1] = alpha[nfcns-2]-alpha[nfcns]
         for j = 2 : nm1
@@ -693,12 +697,12 @@ function remez(numtaps::Integer, band_defs;
                 a[k] = p[k]
                 p[k] = 2.0 * bb * a[k]
             end
-            p[2] += a[1] * 2.0 *aa
+            p[2] = muladd(a[1], 2aa, p[2])
             for k = 1 : j-1
-                p[k] += q[k] + aa * a[k+1]
+                p[k] += muladd(aa, a[k+1], q[k])
             end
             for k = 3 : j+1
-                p[k] += aa * a[k-1]
+                p[k] = muladd(aa, a[k-1], p[k])
             end
 
             if j != nm1
@@ -732,7 +736,7 @@ function remez(numtaps::Integer, band_defs;
             for j = 2 : nm1
                 h[j] = 0.25 * (alpha[nz-j] + alpha[nfcns+2-j])
             end
-            h[nfcns] = 0.5*alpha[1] + 0.25*alpha[2]
+            h[nfcns] = muladd(0.5, alpha[1], 0.25 * alpha[2])
         end
     else
         if nodd
@@ -741,14 +745,14 @@ function remez(numtaps::Integer, band_defs;
             for j = 1 : nm1
                 h[j] = 0.25 * (alpha[nz-j] - alpha[nfcns+3-j])
             end
-            h[nfcns] = 0.5 * alpha[1] - 0.25 * alpha[3]
+            h[nfcns] = muladd(0.5, alpha[1], -0.25 * alpha[3])
             h[nz] = 0.0
         else
             h[1] = 0.25 * alpha[nfcns]
             for j = 2 : nm1
                 h[j] = 0.25 * (alpha[nz-j] - alpha[nfcns+2-j])
             end
-            h[nfcns] = 0.5 * alpha[1] - 0.25 * alpha[2]
+            h[nfcns] = muladd(0.5, alpha[1], -0.25 * alpha[2])
         end
     end
 
